/*
 * Cumulus4j - Securing your data in the cloud - http://cumulus4j.org
 * Copyright (C) 2011 NightLabs Consulting GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.cumulus4j.store;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectStreamClass;

import org.datanucleus.ClassLoaderResolver;

/**
 * A subclass of {@link ObjectInputStream} using the {@link ClassLoaderResolver} to load classes.
 *
 * @author Marco หงุ่ยตระกูล-Schulze - marco at nightlabs dot de
 */
public class DataNucleusObjectInputStream extends java.io.ObjectInputStream
{
	private ClassLoaderResolver classLoaderResolver;

	public DataNucleusObjectInputStream(InputStream in, ClassLoaderResolver classLoaderResolver) throws IOException {
		super(in);

		if (in == null) // in case the super(in) didn't throw an exception, yet, we do it now (fast failure)
			throw new IllegalArgumentException("in == null");

		if (classLoaderResolver == null)
			throw new IllegalArgumentException("classLoaderResolver == null");

		this.classLoaderResolver = classLoaderResolver;
	}

	@Override
	protected Class<?> resolveClass(ObjectStreamClass desc)
	throws IOException, ClassNotFoundException
	{
		return classLoaderResolver.classForName(desc.getName());
	}
}
